/* Data management and preprocessing according to the the grid method 
as described e.g. in 
 J. L. Bentley and J. H. Friedman, Data structures for range searching,
 ACM Computing Surveys, 11 (1079), 397-409.
 
  Author: Oleg Davydov
	  University of Strathclyde
	  Department of Mathematics
          26 Richmond Street
	  Glasgow G1 1XH
	  Scotland, UK
	  e-mail: oleg.davydov@strath.ac.uk


  Copyright (C) 2005 Oleg Davydov

This file is part of TSFIT.

This package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this package; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

 /* structures for data management: grid method  */


typedef struct
{
  int number;
  double xmin, xmax, ymin, ymax, zmin, zmax;
  double scaling_factor;
  double *x, *y, *z;
  int grid_size;
  int *cell_number;
  int **cell_index;
} DATAGRID;

typedef DATAGRID TSFIT_DATA;

/* routines for external use */


void init_tsfit_data (TSFIT_DATA * data, int num, 
                        double *x, double *y, double *z);
void free_tsfit_data (TSFIT_DATA * data);

double scale_x (double x, TSFIT_DATA * data);
double unscale_x (double x, TSFIT_DATA * data);
double scale_y (double y, TSFIT_DATA * data);
double unscale_y (double y, TSFIT_DATA * data);
double scale_z (double z, TSFIT_DATA * data);
double unscale_z (double z, TSFIT_DATA * data);

void init_data_grid (DATAGRID * data);
void free_data_grid (DATAGRID * data);
void find_data_cell (DATAGRID * data, double x, double y, int *i, int *j);
