/* Management of the first stage ("local") methods.
 
  Author: Oleg Davydov
	  University of Strathclyde
	  Department of Mathematics
          26 Richmond Street
	  Glasgow G1 1XH
	  Scotland, UK
	  e-mail: oleg.davydov@strath.ac.uk


  Copyright (C) 2005 Oleg Davydov

This file is part of TSFIT.

This package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this package; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

typedef struct
{
  char type[1000];
  char locale_type[1000];
  void *body;
  void *locale;
  void (*init) (void *body);
  void (*free) (void *body);
  void (*compute) (void *data, void *body);

} TSFIT_LOCAL_METHOD;

void set_local_method_type (TSFIT_LOCAL_METHOD * method, char *type);
void reset_local_method_type (TSFIT_LOCAL_METHOD * method, char *type);

void set_min_points (TSFIT_LOCAL_METHOD * method, int number);
void set_max_points (TSFIT_LOCAL_METHOD * method, int number);
void set_starting_degree (TSFIT_LOCAL_METHOD * method, int degree);
void set_min_sing_value_poly (TSFIT_LOCAL_METHOD * method,
			      double min_sing_value);
void set_radial_basis_function (TSFIT_LOCAL_METHOD * method, int degree,
				double (*basis_function) (double rx, double ry, double delta));
void set_scaling_coef (TSFIT_LOCAL_METHOD * method, double scaling_coef);
void set_min_sing_value_hybrid (TSFIT_LOCAL_METHOD * method,
				double min_sing_value_hybrid);
void set_separation_rbf (TSFIT_LOCAL_METHOD * method, double separ_par);
void set_interpolation_only_rbf (TSFIT_LOCAL_METHOD * method,
				 char *TRUE_or_FALSE);
