/* Range searching according to the the grid method as described e.g. in 
 J. L. Bentley and J. H. Friedman, Data structures for range searching,
 ACM Computing Surveys, 11 (1079), 397-409.
 
  Author: Oleg Davydov
	  University of Strathclyde
	  Department of Mathematics
          26 Richmond Street
	  Glasgow G1 1XH
	  Scotland, UK
	  e-mail: oleg.davydov@strath.ac.uk


  Copyright (C) 2005 Oleg Davydov

This file is part of TSFIT.

This package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this package; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* local search */

typedef struct
{
  int max_number;		/* maximum number of points (used for memory allocation);
				   MUST be a SQUARE number if "most central" thinning is used */
  int number;			/* actual number of local points used for the computation */
  double *x, *y, *z;
  /* auxiliary numbers and arrays for thinning */
  int loc_grid_sizeX;
  int *loc_grid;
  double *loc_grid_dist;
} LOCALDATA;



typedef struct
{
  LOCALDATA *locPoints;		/* pointer to the local data */
  double diam;			/* diameter of the local disk where the points are actually taken */
  /* user specified parameters for point selection */
  int minPoints;
  int maxPoints;
  double init_rad_factor;	/* quotient (initial radius)/(x-side of spline triangle) */
} LSCONTROL;


/* routines for the external use */

void choose_loc_points (TSFIT_DATA * data, LSCONTROL * lscontr,
			double mx, double my, double loc_size);
void init_loc_search (LSCONTROL * lcontr);
void free_loc_search (LSCONTROL * lcontr);
